c
c-----------------------------------------------------------------------
c
c ****** Calculate the closest distance to coronal hole boundaries.
c
c-----------------------------------------------------------------------
c
      module hdflib
c
c-----------------------------------------------------------------------
c
c ****** Flag to indicate whether the HDF library has been compiled
c ****** using 32-bit floats (HDFLIB32=.true.) or 64-bit floats
c ****** (HDFLIB32=.false.).
c
c ****** For example, on the Crays, set HDFLIB32=.false., since
c ****** the HDF library has been compiled to handle floats in
c ****** the default real type, REAL(8).
c
c ****** On machines on which the default float precision is REAL(4),
c ****** (e.g., SGI, Linux), set HDFLIB32=.true. if the HDF
c ****** library has been compiled using the default real
c ****** type, REAL(4).
c
      logical, parameter :: HDFLIB32=.true.
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module sds_def
c
c-----------------------------------------------------------------------
c
c ****** Definition of the SDS data structure.
c
      type :: sds
        integer :: ndim
        integer :: nx,ny,nz
        logical :: scale
        real, dimension(:), pointer :: x,y,z
        real, dimension(:,:,:), pointer :: fld
      end type
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module rdhdf_2d_interface
        interface
          subroutine rdhdf_2d (fname,scale,nx,ny,f,x,y,ierr)
          character(*) :: fname
          logical :: scale
          integer :: nx,ny
          real, dimension(:,:), pointer :: f
          real, dimension(:), pointer :: x,y
          integer :: ierr
          end
        end interface
      end module
c
c-----------------------------------------------------------------------
c
      module wrhdf_2d_interface
        interface
          subroutine wrhdf_2d (fname,scale,nx,ny,f,x,y,ierr)
          use sds_def
          character(*) :: fname
          logical :: scale
          integer :: nx,ny
          real, dimension(nx,ny,1), target :: f
          real, dimension(nx), target :: x
          real, dimension(ny), target :: y
          integer :: ierr
          end
        end interface
      end module
c
c-----------------------------------------------------------------------
c
      program CH_DIST
c
c-----------------------------------------------------------------------
c
      use rdhdf_2d_interface
      use wrhdf_2d_interface
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage.
c
      integer :: nt,np
c
      real, dimension(:), pointer :: t,p
      real, dimension(:,:), pointer :: r1
c
      real, dimension(:,:), pointer :: d,x,y,z
c
c-----------------------------------------------------------------------
c
      logical scale
      character(256) :: fname
c
c-----------------------------------------------------------------------
c
c ****** Read the coronal hole file.
c
      write (*,*)
      write (*,'(1x,a,$)') 'Coronal hole (t,p) HDF input file: '
      read (*,'(a)') fname
c
      call rdhdf_2d (fname,scale,nt,np,r1,t,p,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in CH_DIST:'
        write (*,*) '### Could not read the coronal hole file.'
        write (*,*) 'File name: ',trim(fname)
        call exit (1)
      end if
c
      if (.not.scale) then
        write (*,*)
        write (*,*) '### ERROR in CH_DIST:'
        write (*,*) '### The coronal hole file does not have scales.'
        write (*,*) 'File name: ',trim(fname)
        call exit (1)
      end if
c
      write (*,*)
      write (*,'(1x,a,$)') 'Coronal hole distance HDF output file: '
      read (*,'(a)') fname
c
c ****** Allocate storage.
c
      allocate (d(nt,np))
      allocate (x(nt,np))
      allocate (y(nt,np))
      allocate (z(nt,np))
c
c ****** Precompute the Cartesian coordinates of each point
c ****** for efficiency.
c
      do i=1,nt
        do j=1,np
          call s2c (1.,t(i),p(j),x(i,j),y(i,j),z(i,j))
        enddo
      enddo
c
c ****** Loop over all points to find the closest distance to
c ****** the coronal hole boundary.
c
      rmin=1.01
c
      do i=1,nt
        do j=1,np
c
          x0=x(i,j)
          y0=y(i,j)
          z0=z(i,j)
c
          if (r1(i,j).lt.rmin) then
c
c ****** This point is not in a coronal hole.
c
            xdxmx=-1.
            do ii=1,nt
              do jj=1,np
                if ((.not.(i.eq.ii.and.j.eq.jj)).and.
     &              r1(ii,jj).ge.rmin) then
                  xdotx0=x0*x(ii,jj)+y0*y(ii,jj)+z0*z(ii,jj)
                  xdxmx=max(xdxmx,xdotx0)
                end if
              enddo
            enddo
            xdxmx=max(xdxmx,-1.)
            xdxmx=min(xdxmx,1.)
            d(i,j)=acos(xdxmx)
c
          else
c
c ****** This point is in a coronal hole.
c
            xdxmx=-1.
            do ii=1,nt
              do jj=1,np
                if ((.not.(i.eq.ii.and.j.eq.jj)).and.
     &              r1(ii,jj).lt.rmin) then
                  xdotx0=x0*x(ii,jj)+y0*y(ii,jj)+z0*z(ii,jj)
                  xdxmx=max(xdxmx,xdotx0)
                end if
              enddo
            enddo
            d(i,j)=-acos(xdxmx)
c
          end if
c
        enddo
      enddo
c
c ****** Write the coronal hole distance to a file.
c
      call wrhdf_2d (fname,.true.,nt,np,d,t,p,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in CH_DIST:'
        write (*,*) '### Could not write the coronal hole distance '//
     &              'output file.'
        write (*,*) 'File name: ',trim(fname)
        call exit (1)
      end if
c
      write (*,*)
      write (*,*) '### Comment from CH_DIST:'
      write (*,*) 'Wrote coronal hole distance to file: ',trim(fname)
c
      call exit (0)
      end
c#######################################################################
      subroutine s2c (r,t,p,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Convert from spherical coordinates (R,T,P) 
c ****** to Cartesian coordinates (X,Y,Z).
c
c ****** This routine assumes that T and P are in radians.
c
c-----------------------------------------------------------------------
c
      st=sin(t)
      x=r*st*cos(p)
      y=r*st*sin(p)
      z=r*cos(t)
c
      return
      end
c#######################################################################
      subroutine rdhdf_2d (fname,scale,nx,ny,f,x,y,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a 2D scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine calls routine RDHDF to read the file.
c
c-----------------------------------------------------------------------
c
      use sds_def
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      logical :: scale
      integer :: nx,ny
      real, dimension(:,:), pointer :: f
      real, dimension(:), pointer :: x,y
      integer :: ierr
      intent(in) :: fname
      intent(out) :: scale,nx,ny,ierr
c
c-----------------------------------------------------------------------
c
c ****** Declaration for the SDS structure.
c
      type(sds) :: s
c
c-----------------------------------------------------------------------
c
c ****** Read the data set.
c
      call rdhdf (fname,s,ierr)
c
      if (ierr.ne.0) return
c
c ****** Check that this is a 2D data set.
c
      if (s%ndim.ne.2) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF_2D:'
        write (*,*) '### The HDF file does not contain a 2D data set.'
        write (*,*) 'File name: ',trim(fname)
        ierr=3
        return
      end if
c
c ****** Set the output arguments.
c
      nx=s%nx
      ny=s%ny
      scale=s%scale
      x=>s%x
      y=>s%y
      f=>s%fld(:,:,1)
c
      return
      end  
c#######################################################################
      subroutine wrhdf_2d (fname,scale,nx,ny,f,x,y,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a 2D scientific data set to an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine calls routine WRHDF to write the file.
c
c-----------------------------------------------------------------------
c
      use sds_def
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      logical :: scale
      integer :: nx,ny
      real, dimension(nx,ny,1), target :: f
      real, dimension(nx), target :: x
      real, dimension(ny), target :: y
      integer :: ierr
      intent(in) :: fname,scale,nx,ny,f,x,y
      intent(out) :: ierr
c
c-----------------------------------------------------------------------
c
c ****** Declaration for the SDS structure.
c
      type(sds) :: s
c
c-----------------------------------------------------------------------
c
c ****** Set the structure components.
c
      s%ndim=2
      s%nx=nx
      s%ny=ny
      s%nz=1
      s%scale=scale
      if (scale) then
        s%x=>x
        s%y=>y
      else
        nullify (s%x)
        nullify (s%y)
      end if
      nullify (s%z)
      s%fld=>f
c
c ****** Write the data set.
c
      call wrhdf (fname,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in WRHDF_2D:'
        write (*,*) '### Could not write the 2D data set.'
        write (*,*) 'File name: ',trim(fname)
        return
      end if
c
      return
      end  
c#######################################################################
      subroutine rdhdf (fname,s,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a 1D, 2D, or 3D scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scale arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character(*)]
c                    HDF data file name to read from.
c
c ****** Output arguments:
c
c          S       : [structure of type SDS]
c                    A structure that holds the field, its
c                    dimensions, and the scales, with the
c                    components described below.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c ****** Components of structure S:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          NX      : [integer]
c                    Number of points in the first dimension.
c
c          NY      : [integer]
c                    Number of points in the second dimension.
c                    For a 1D data set, NY=1 is returned.
c
c          NZ      : [integer]
c                    Number of points in the third dimension.
c                    For a 1D or 2D data set, NZ=1 is returned.
c
c          SCALE   : [logical]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  SCALE=.false. means that scales
c                    were not found; SCALE=.true. means that scales
c                    were found.
c
c          X       : [real, pointer to a rank-1 array of 
c                     dimension (NX)]
c                    This array holds the scale for the first
c                    dimension when SCALE=.true., and is undefined
c                    otherwise.
c
c          Y       : [real, pointer to a rank-1 array of 
c                     dimension (NY)]
c                    This array holds the scale for the second
c                    dimension (if present) when SCALE=.true.,
c                    and is undefined otherwise.
c
c          Z       : [real, pointer to a rank-1 array of 
c                     dimension (NZ)]
c                    This array holds the scale for the third
c                    dimension (if present) when SCALE=.true.,
c                    and is undefined otherwise.
c
c          FLD     : [real, pointer to a rank-3 array of 
c                     dimension (NX,NY,NZ)]
c                    This array holds the data set values.
c
c ****** The storage for the arrays pointed to by FLD, and the
c ****** scales X, Y, and Z (if present), is allocated by this
c ****** routine.
c
c-----------------------------------------------------------------------
c
      use sds_def
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      type(sds) :: s
      integer :: ierr
      intent(in) :: fname
      intent(out) :: s,ierr
c
c-----------------------------------------------------------------------
c
c ****** Local variables.
c
      integer :: ndim
      integer :: nx,ny,nz
      logical :: scale
      real, dimension(:), pointer :: x,y,z
      real, dimension(:,:,:), pointer :: fld
c
c-----------------------------------------------------------------------
c
c ****** Declarations for temporary variables.
c
      integer, parameter :: mxdim=3
      integer, dimension(mxdim) :: idim
c
      integer :: iret,nscal
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dsgdims,DFSDrestart
      external dsgdims,DFSDrestart
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Reset the HDF read routines to read from the beginning
c ****** of the file.  This is required, since, if the previous
c ****** read happened to be from the same file, the next 
c ****** read will be from the end of the last data set read,
c ****** and not the beginning of the file.
c
      iret=DFSDrestart()
c
c ****** Get the dimensions of the SDS.
c
c ****** Due to a bug in the HDF library (versions 3.3r4 and 4.0r2),
c ****** set NDIM to zero to prevent a core dump if the file is
c ****** missing.
c
      ndim=0
c
      iret=dsgdims(fname,ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',trim(fname)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
      nx=idim(1)
      ny=1
      nz=1
      if (ndim.ge.2) ny=idim(2)
      if (ndim.ge.3) nz=idim(3)
c
c ****** Get the scales.
c
      nscal=0
c
      allocate (x(nx))
      iret=ispdp_dsgdisc(1,nx,x)
      if (iret.eq.0) nscal=nscal+1
c
      if (ndim.ge.2) then
        allocate (y(ny))
        iret=ispdp_dsgdisc(2,ny,y)
        if (iret.eq.0) nscal=nscal+1
      end if
c
      if (ndim.ge.3) then
        allocate (z(nz))
        iret=ispdp_dsgdisc(3,nz,z)
        if (iret.eq.0) nscal=nscal+1
      end if
c
c ****** Check if all the scales are present.
c
      if (nscal.eq.ndim) then
        scale=.true.
      else
        scale=.false.
      end if
c
c ****** If scales are not present, deallocate any scale arrays 
c ****** that may have been allocated.
c
      if (.not.scale) then
        deallocate (x)
        nullify (x)
        if (ndim.ge.2) then
          deallocate (y)
          nullify (y)
        end if
        if (ndim.ge.3) then
          deallocate (z)
          nullify (z)
        end if
      end if
c
c ****** Allocate the memory for the array.
c
      allocate (fld(nx,ny,nz))
c
c ****** Read the data array.
c
      n=nx*ny*nz
      iret=ispdp_dsgdata(fname,ndim,idim,n,fld)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',trim(fname)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=2
        return
      end if
c
c ****** Load the output structure from the local variables.
c
      s%ndim=ndim
      s%nx=nx
      s%ny=ny
      s%nz=nz
      s%scale=scale
      s%x=>x
      s%y=>y
      s%z=>z
      s%fld=>fld
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,s,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a 1D, 2D, or 3D scientific data set to an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character(*)]
c                    HDF data file name to write to.
c
c          S       : [structure of type SDS]
c                    A structure that holds the field, its
c                    dimensions, and the scales, with the
c                    components described below.
c
c ****** Output arguments:
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was written
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c ****** Components of structure S:
c
c          NDIM    : [integer]
c                    Number of dimensions in the data set.
c
c          NX      : [integer]
c                    Number of points in the first dimension.
c
c          NY      : [integer]
c                    Number of points in the second dimension.
c                    NY is ignored when NDIM.lt.2.
c
c          NZ      : [integer]
c                    Number of points in the third dimension.
c                    NZ is ignored when NDIM.lt.3.
c
c          SCALE   : [logical]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  SCALE=.false. means that scales
c                    are not being supplied; SCALE=.true. means that
c                    scales are being supplied.
c
c          X       : [real, pointer to a rank-1 array of 
c                     dimension (NX)]
c                    This array holds the scale for the first
c                    dimension when SCALE=.true., and is not
c                    referenced otherwise.
c
c          Y       : [real, pointer to a rank-1 array of 
c                     dimension (NY)]
c                    This array holds the scale for the second
c                    dimension (if present) when SCALE=.true.,
c                    and is not referenced otherwise.
c
c          Z       : [real, pointer to a rank-1 array of 
c                     dimension (NZ)]
c                    This array holds the scale for the third
c                    dimension (if present) when SCALE=.true.,
c                    and is not referenced otherwise.
c
c          FLD     : [real, pointer to a rank-3 array of 
c                     dimension (NX,NY,NZ)]
c                    This array holds the data set values.
c
c-----------------------------------------------------------------------
c
      use sds_def
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      type(sds) :: s
      integer :: ierr
      intent(in) :: fname,s
      intent(out) :: ierr
c
c-----------------------------------------------------------------------
c
c ****** Local variables.
c
      integer :: ndim
      integer :: nx,ny,nz
      logical :: scale
      real, dimension(:), pointer :: x,y,z
      real, dimension(:,:,:), pointer :: fld
c
c-----------------------------------------------------------------------
c
c ****** Declarations for temporary variables.
c
      integer, parameter :: mxdim=3
      integer, dimension(mxdim) :: idim
c
      integer :: iret
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dssdims
      external dssdims
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Load the local variables from the input structure.
c
      ndim=s%ndim
      nx=s%nx
      ny=s%ny
      nz=s%nz
      scale=s%scale
      if (scale) then
        x=>s%x
        if (ndim.ge.2) y=>s%y
        if (ndim.ge.3) z=>s%z
      else
        nullify (x)
        nullify (y)
        nullify (z)
      end if
      fld=>s%fld
c
c ****** Set the dimensions.
c
      idim(1)=nx
      idim(2)=1
      idim(3)=1
      if (ndim.ge.2) idim(2)=ny
      if (ndim.ge.3) idim(3)=nz
c
      iret=dssdims(ndim,idim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in WRHDF:'
        write (*,*) '### Could not set the SDS dimensions.'
        write (*,*) 'File name: ',trim(fname)
        write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Set the scales.
c
      if (scale) then
c
        iret=ispdp_dssdisc(1,nx,x)
        if (iret.ne.0) go to 900
c
        if (ndim.ge.2) then
          iret=ispdp_dssdisc(2,ny,y)
          if (iret.ne.0) go to 900
        end if
c
        if (ndim.ge.3) then
          iret=ispdp_dssdisc(3,nz,z)
          if (iret.ne.0) go to 900
        end if
c
      end if
c
c ****** Write the data array.
c
      n=nx*ny*nz
      iret=ispdp_dspdata(fname,ndim,idim,n,fld)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in WRHDF:'
        write (*,*) '### Could not write the SDS data.'
        write (*,*) 'File name: ',trim(fname)
        write (*,*) '[Error return (from DSPDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',trim(fname)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      ierr=2
c
      return
      end
c#######################################################################
      integer function ispdp_dsgdisc (idim,n,f)
c
c-----------------------------------------------------------------------
c
c ****** Wrapper routine to call the HDF routine DSGDISC.
c
c ****** This routine takes into account whether the HDF library
c ****** has been compiled for 32-bit floats or 64-bit floats,
c ****** according to the setting of the flag HDFLIB32
c ****** (in module HDFLIB).
c
c-----------------------------------------------------------------------
c
      use hdflib
c
c-----------------------------------------------------------------------
c
      integer :: idim,n
      real, dimension(n) :: f
c
c-----------------------------------------------------------------------
c
      real(4), dimension(n) :: f4
      integer :: iret
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dsgdisc
      external dsgdisc
c
c-----------------------------------------------------------------------
c
      if (HDFLIB32.and.kind(f).ne.4) then
        iret=dsgdisc(idim,n,f4)
        f(:)=f4(:)
      else
        iret=dsgdisc(idim,n,f)
      end if
c
      ispdp_dsgdisc=iret
c
      return
      end
c#######################################################################
      integer function ispdp_dsgdata (fname,ndim,idim,n,f)
c
c-----------------------------------------------------------------------
c
c ****** Wrapper routine to call the HDF routine DSGDATA.
c
c ****** This routine takes into account whether the HDF library
c ****** has been compiled for 32-bit floats or 64-bit floats,
c ****** according to the setting of the flag HDFLIB32
c ****** (in module HDFLIB).
c
c-----------------------------------------------------------------------
c
      use hdflib
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      integer :: ndim
      integer, dimension(ndim) :: idim
      integer :: n
      real, dimension(n) :: f
c
c-----------------------------------------------------------------------
c
      real(4), dimension(n) :: f4
      integer :: iret
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dsgdata
      external dsgdata
c
c-----------------------------------------------------------------------
c
      if (HDFLIB32.and.kind(f).ne.4) then
        iret=dsgdata(fname,ndim,idim,f4)
        f(:)=f4(:)
      else
        iret=dsgdata(fname,ndim,idim,f)
      end if
c
      ispdp_dsgdata=iret
c
      return
      end
c#######################################################################
      integer function ispdp_dssdisc (idim,n,f)
c
c-----------------------------------------------------------------------
c
c ****** Wrapper routine to call the HDF routine DSSDISC.
c
c ****** This routine takes into account whether the HDF library
c ****** has been compiled for 32-bit floats or 64-bit floats,
c ****** according to the setting of the flag HDFLIB32
c ****** (in module HDFLIB).
c
c-----------------------------------------------------------------------
c
      use hdflib
c
c-----------------------------------------------------------------------
c
      integer :: idim,n
      real, dimension(n) :: f
c
c-----------------------------------------------------------------------
c
      real(4), dimension(n) :: f4
      integer :: iret
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dssdisc
      external dssdisc
c
c-----------------------------------------------------------------------
c
      if (HDFLIB32.and.kind(f).ne.4) then
        f4(:)=f(:)
        iret=dssdisc(idim,n,f4)
      else
        iret=dssdisc(idim,n,f)
      end if
c
      ispdp_dssdisc=iret
c
      return
      end
c#######################################################################
      integer function ispdp_dspdata (fname,ndim,idim,n,f)
c
c-----------------------------------------------------------------------
c
c ****** Wrapper routine to call the HDF routine DSPDATA.
c
c ****** This routine takes into account whether the HDF library
c ****** has been compiled for 32-bit floats or 64-bit floats,
c ****** according to the setting of the flag HDFLIB32
c ****** (in module HDFLIB).
c
c-----------------------------------------------------------------------
c
      use hdflib
c
c-----------------------------------------------------------------------
c
      character(*) :: fname
      integer :: ndim
      integer, dimension(ndim) :: idim
      integer :: n
      real, dimension(n) :: f
c
c-----------------------------------------------------------------------
c
      real(4), dimension(n) :: f4
      integer :: iret
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer :: dspdata
      external dspdata
c
c-----------------------------------------------------------------------
c
      if (HDFLIB32.and.kind(f).ne.4) then
        f4(:)=f(:)
        iret=dspdata(fname,ndim,idim,f4)
      else
        iret=dspdata(fname,ndim,idim,f)
      end if
c
      ispdp_dspdata=iret
c
      return
      end
