c
c-----------------------------------------------------------------------
c
c ****** Extract a subset of scientific data set. 
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        10/12/92, ZM, Version 1.00:
c
c         - Original version of program.
c
c        02/06/93, ZM, Version 1.01:
c
c         - Added the -mp option to interpret coordinates
c           as mesh points.  This allows the extraction region
c           to be specified in mesh points.
c
c        04/07/93, ZM, Version 1.02:
c
c         - Added the ability to specify a stride for the extraction
c           region.
c
c        07/29/2003, ZM, Version 1.03:
c
c         - This version uses the ZM tools libraries.
c         - The program thus uses the new HDF read/write
c           routines that can handle 32-bit and 64-bit data files.
c
c-----------------------------------------------------------------------
c
c#######################################################################
c
c ****** Note that this tool uses modules from ZM's tools library.
c
c#######################################################################
      module params
c
c-----------------------------------------------------------------------
c ****** Parameters.
c-----------------------------------------------------------------------
c
      use number_types
c
      implicit none
c
      character(256) :: infile
      character(256) :: outfile
      character(16) :: fmt
c
      logical :: verbose
      logical :: mpmode
      logical :: hdf_precision_set=.false.
      logical :: hdf32
      logical :: keepdims
c
      integer, parameter :: mxdim=3
      logical, dimension(mxdim) :: setlim0,setlim1
      integer, dimension(mxdim) :: ilim0,ilim1
      real(r_typ), dimension(mxdim) :: xlim0,xlim1
      integer, dimension(mxdim) :: i0,i1,is
c
      end module
c#######################################################################
      program EXTRACT
c
c-----------------------------------------------------------------------
c
      use params
      use sds_def
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='EXTRACT'
      character(*), parameter :: cvers='1.03'
      character(*), parameter :: cdate='07/29/2003'
c
c-----------------------------------------------------------------------
c
      character, dimension(3), parameter :: label=(/'X','Y','Z'/)
c
c-----------------------------------------------------------------------
c
      type(sds) :: s,sx
      integer :: ierr,i
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call set_parameters (cname,cvers,cdate)
c
c ****** Read the SDS.
c
      call rdsds (fmt,infile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the data set.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',trim(infile)
        call exit (1)
      end if
c
      if ((.not.mpmode).and..not.s%scale) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### The data set does not contain scales.'
        write (*,*) 'Use option -mp to specify the extraction '//
     &              'region using mesh points.'
        write (*,*) 'File name: ',trim(infile)
        call exit (1)
      end if
c
c ****** Set the extraction region.
c
      call set_extraction_region (s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Invalid extraction region specified.'
        write (*,*) 'File name: ',trim(infile)
        call exit (1)
      end if
c
c ****** Extract the desired subset.
c
      call extract_region (i0,i1,is,keepdims,s,sx)
c
c ****** Set the precision for the output SDS.
c
      if (hdf_precision_set) then
        sx%hdf32=hdf32
      else
        sx%hdf32=s%hdf32
      end if
c
c ****** Write the subset.
c
      call wrsds (fmt,outfile,sx,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write the data subset.'
        write (*,*) 'IERR (from WRSDS) = ',ierr
        write (*,*) 'File name: ',trim(outfile)
        call exit (1)
      end if
c
c ****** Write diagnostics.
c
      if (verbose) then
        if (.not.mpmode) then
          write (*,*)
          write (*,*) 'Extracted the region:'
          do i=1,s%ndim
            write (*,900) label(i)//': ',s%scales(i)%f(i0(i)),
     &                                   s%scales(i)%f(i1(i))
          enddo
        end if
        write (*,*)
        write (*,*) 'Mesh indices of extracted region:'
        do i=1,s%ndim
          write (*,910) label(i)//': ',i0(i),i1(i),is(i)
        enddo
        write (*,*)
        write (*,920) 'Output SDS contains a ',sx%ndim,
     &                'D data set with dimensions: ',
     &                (sx%dims(i),i=1,sx%ndim)
      end if
c
  900 format (1x,a,'[',1pe13.6,',',1pe13.6,']')
  910 format (1x,a,'[',i8,',',i8,']',', stride = ',i8)
  920 format (1x,a,i1,a,3i8)
c
      call exit (0)
c
      end
c#######################################################################
      subroutine set_extraction_region (s,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Set the mesh point limits for the extraction region
c ****** specified.
c
c-----------------------------------------------------------------------
c
      use number_types
      use sds_def
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      type(sds) :: s
      integer :: ierr
c
c-----------------------------------------------------------------------
c
      integer :: i
      real(r_typ) :: x0,x1
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      i0(:)=1
      i1(:)=1
c
      if (mpmode) then
c
c ****** In mesh-point-mode, set the mesh points directly.
c
        do i=1,s%ndim
          if (setlim0(i)) then
            i0(i)=ilim0(i)
          else
            i0(i)=1
          end if
          if (setlim1(i)) then
            i1(i)=ilim1(i)
          else
            i1(i)=s%dims(i)
          end if
          if (i0(i).lt.1.or.i0(i).gt.i1(i).or.
     &        i1(i).gt.s%dims(i)) go to 900
        enddo
c
        return
c
      else
c
c ****** In real-coordinates mode, deduce the mesh-point limits.
c
        do i=1,s%ndim
          if (setlim0(i)) then
            x0=xlim0(i)
          else
            x0=s%scales(i)%f(1)
          end if
          if (setlim1(i)) then
            x1=xlim1(i)
          else
            x1=s%scales(i)%f(s%dims(i))
          end if
          call set_limits (s%dims(i),s%scales(i)%f,x0,x1,
     &                     i0(i),i1(i),ierr)
          if (ierr.ne.0) return
        enddo
c
      end if
c
      return
c
  900 continue
c
c ****** Error: invalid region specified.
c
      write (*,*)
      write (*,*) '### ERROR in SET_EXTRACTION_REGION:'
      write (*,*) '### Invalid extraction region specified.'
      write (*,*) '### The specified range is invalid:'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Number of mesh points = ',s%dims(i)
      write (*,*) 'Mesh point limits specified: ',i0(i),i1(i)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine set_limits (nx,x,x0,x1,i0,i1,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Find the mesh point indices I0 and I1 in array X(NX) that
c ****** are enclosed by the segment [X0,X1].
c
c-----------------------------------------------------------------------
c
      use number_types
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      integer :: nx
      real(r_typ), dimension(nx) :: x
      real(r_typ) :: x0,x1
      integer :: i0,i1
      integer :: ierr
c
c-----------------------------------------------------------------------
c
      real(r_typ), parameter :: half=.5_r_typ
c
c-----------------------------------------------------------------------
c
      integer :: i
      real(r_typ) :: xav
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      if (x1.lt.x0) go to 900
c
      do i=1,nx
        if (x(i).ge.x0) exit
      enddo
      if (i.gt.nx) go to 900
      i0=i
c
      do i=nx,1,-1
        if (x(i).le.x1) exit
      enddo
      if (i.lt.1) go to 900
      i1=i
c
c ****** Check that at least one mesh point is included.
c
      if (i1.lt.i0) then
        xav=half*(x0+x1)
        if (abs(x(i0)-xav).le.abs(x(i1)-xav)) then
          i1=i0
        else
          i0=i1
        end if
      end if
c
      return
c
  900 continue
c
c ****** Error: invalid region specified.
c
      write (*,*)
      write (*,*) '### ERROR in SET_LIMITS:'
      write (*,*) '### Invalid extraction region specified:'
      write (*,*) 'Data limits:'
      write (*,*) 'Min = ',x(1)
      write (*,*) 'Max = ',x(nx)
      write (*,*) 'Extraction region:'
      write (*,*) 'Min = ',x0
      write (*,*) 'Max = ',x1
      ierr=1
c
      return
      end
c#######################################################################
      subroutine extract_region (i0,i1,is,keepdims,s,sx)
c
c-----------------------------------------------------------------------
c
c ****** Extract the subregion specified by I0, I1, and IS
c ****** from data set S to data set SX.
c
c-----------------------------------------------------------------------
c
      use number_types
      use sds_def
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      integer, dimension(3) :: i0,i1,is
      type(sds) :: s
      type(sds) :: sx
      logical :: keepdims
c
c-----------------------------------------------------------------------
c
      integer, dimension(3) :: dims,map
      integer :: ndim,i,np
c
c-----------------------------------------------------------------------
c
c ****** If KEEPDIMS=.true., maintain the dimensionality of the
c ****** extracted data set.
c
      dims(:)=1
      map(:)=1
c
      if (keepdims) then
c
        ndim=s%ndim
        do i=1,ndim
          dims(i)=1+(i1(i)-i0(i))/is(i)
          map(i)=i
        enddo
c
      else
c
c ****** Create the list of dimensions for the subset.
c ****** Note that dimensions of length 1 are deleted.
c
        ndim=0
        do i=1,s%ndim
          np=1+(i1(i)-i0(i))/is(i)
          if (np.gt.1) then
            ndim=ndim+1
            dims(ndim)=np
            map(ndim)=i
          end if
        enddo
c
c ****** Special case: only one datum value.
c
        if (ndim.eq.0) then
          ndim=1
          map(1)=1
        end if
c
      end if
c
      sx%ndim=ndim
      sx%dims(:)=dims(:)
      sx%scale=s%scale
c
c ****** Use FORTRAN90 array syntax to specify the extracted subset.
c
      allocate (sx%f(dims(1),dims(2),dims(3)))
c
      sx%f=reshape(s%f(i0(1):i1(1):is(1),
     &                 i0(2):i1(2):is(2),
     &                 i0(3):i1(3):is(3)),
     &             (/dims(1),dims(2),dims(3)/))
c
      if (s%scale) then
        do i=1,ndim
          allocate (sx%scales(i)%f(dims(i)))
          sx%scales(i)%f=s%scales(map(i))%f(i0(map(i)):
     &                                      i1(map(i)):
     &                                      is(map(i)))
        enddo
        do i=ndim+1,3
          allocate (sx%scales(i)%f(1))
        enddo
      else
        allocate (sx%scales(1)%f(1))
        allocate (sx%scales(2)%f(1))
        allocate (sx%scales(3)%f(1))
      end if
c
      return
      end
c#######################################################################
      subroutine set_parameters (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use number_types
      use syntax
      use paragraph_def
      use get_usage_line_interface
      use print_par_interface
      use delete_par_interface
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*) :: cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Storage the for usage line.
c
      type(paragraph), pointer :: usage
c
c ****** Storage for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      integer :: ierr
      character(256) :: arg
      logical :: set
c
c-----------------------------------------------------------------------
c
      integer, external :: intval
      real(r_typ), external :: fpval
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (GROUP_K ,'-v',' ',' ')
      call defarg (GROUP_KA,'-fmt','hdf','text|hdf')
      call defarg (GROUP_KA,'-x0','default','<val>')
      call defarg (GROUP_KA,'-x1','default','<val>')
      call defarg (GROUP_KA,'-y0','default','<val>')
      call defarg (GROUP_KA,'-y1','default','<val>')
      call defarg (GROUP_KA,'-z0','default','<val>')
      call defarg (GROUP_KA,'-z1','default','<val>')
      call defarg (GROUP_KA,'-dx','1','<x-stride>')
      call defarg (GROUP_KA,'-dy','1','<y-stride>')
      call defarg (GROUP_KA,'-dz','1','<z-stride>')
      call defarg (GROUP_K ,'-mp',' ',' ')
      call defarg (GROUP_A ,'infile',' ',' ')
      call defarg (GROUP_A ,'outfile',' ',' ')
      call defarg (GROUP_K ,'-32',' ',' ')
      call defarg (GROUP_K ,'-64',' ',' ')
      call defarg (GROUP_K ,'-keepdims',' ',' ')
c
c ****** Parse the command line.
c
      call parse (errmsg,ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Extract a subset from a data set.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage line.
c
        call get_usage_line (usage)
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
c
        call print_par (usage)
c
        write (*,*)
        write (*,*) '-fmt specifies the file format (default=hdf).'
        write (*,*) 'Use -x0 and -x1 ... to extract data in the '//
     &              'specified coordinate range.'
        write (*,*) 'Use -dx ... to set the strides '// 
     &              '(in mesh points) for the extraction region.'
        write (*,*) 'Use -mp to interpret the specified '// 
     &              'coordinates as mesh points.'
        write (*,*) 'The number of bits to use for the extracted'//
     &              ' HDF file can be specified'
        write (*,*) 'using -64 or -32 (default=same as input file).'
        write (*,*) '-keepdims can be used to keep'//
     &              ' the dimensionality of the data set (i.e., a 3D'
        write (*,*) 'data set remains 3D even if some of its'//
     &              ' dimensions may be of length 1).'
c
        call delete_par (usage)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Verbose flag.
c
      call fetcharg ('-v',set,arg)
      verbose=set
c
c ****** Mesh point coordinates flag.
c
      call fetcharg ('-mp',set,arg)
      mpmode=set
c
c ****** File format.
c
      call fetcharg ('-fmt',set,arg)
      fmt=trim(arg)
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      infile=trim(arg)
c
c ****** Output file name.
c
      call fetcharg ('outfile',set,arg)
      outfile=trim(arg)
c
c ****** Extraction region.
c
      call fetcharg ('-x0',set,arg)
      setlim0(1)=set
      if (set) then
        if (mpmode) then
          ilim0(1)=intval(trim(arg),'-x0')
        else
          xlim0(1)=fpval(trim(arg),'-x0')
        end if
      end if
c
      call fetcharg ('-x1',set,arg)
      setlim1(1)=set
      if (set) then
        if (mpmode) then
          ilim1(1)=intval(trim(arg),'-x1')
        else
          xlim1(1)=fpval(trim(arg),'-x1')
        end if
      end if
c
      call fetcharg ('-y0',set,arg)
      setlim0(2)=set
      if (set) then
        if (mpmode) then
          ilim0(2)=intval(trim(arg),'-y0')
        else
          xlim0(2)=fpval(trim(arg),'-y0')
        end if
      end if
c
      call fetcharg ('-y1',set,arg)
      setlim1(2)=set
      if (set) then
        if (mpmode) then
          ilim1(2)=intval(trim(arg),'-y1')
        else
          xlim1(2)=fpval(trim(arg),'-y1')
        end if
      end if
c
      call fetcharg ('-z0',set,arg)
      setlim0(3)=set
      if (set) then
        if (mpmode) then
          ilim0(3)=intval(trim(arg),'-z0')
        else
          xlim0(3)=fpval(trim(arg),'-z0')
        end if
      end if
c
      call fetcharg ('-z1',set,arg)
      setlim1(3)=set
      if (set) then
        if (mpmode) then
          ilim1(3)=intval(trim(arg),'-z1')
        else
          xlim1(3)=fpval(trim(arg),'-z1')
        end if
      end if
c
c ****** Set the strides.
c
      call fetcharg ('-dx',set,arg)
      is(1)=intval(trim(arg),'-dx')
c
      call fetcharg ('-dy',set,arg)
      is(2)=intval(trim(arg),'-dy')
c
      call fetcharg ('-dz',set,arg)
      is(3)=intval(trim(arg),'-dz')
c
c ****** Precision for the HDF output file.
c
      call fetcharg ('-32',set,arg)
      if (set) then
        hdf_precision_set=.true.
        hdf32=.true.
      end if
c
      call fetcharg ('-64',set,arg)
      if (set) then
        if (hdf_precision_set) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Only one output precision option can'//
     &                ' be secified: -32 or -64'
          call exit (1)
        end if
        hdf_precision_set=.true.
        hdf32=.false.
      end if
c
c ****** Switch to maintain dimensionality.
c
      call fetcharg ('-keepdims',set,arg)
      keepdims=set
c
      return
      end
