;===============================================================
;+
; NAME:
;    position_set
; PURPOSE:
;    Setup position of a plot
; CATEGORY:
;    Graphics
; CALLING SEQUENCE:
;    position_set, x, y, pxx, pyy
; INPUTS:
;    None
; INPUT KEYWORD PARAMETERS:
;    xrange:     Minimum and maximum x-positions
;    yrange:     Minimum and maximum y-positions
;    pxrange:    Allocated X-positions [x0,x1] for plotting area
;    pyrange:    Allocated Y-positions [y0,y1] for plotting area
;    angle:      Requested rotation of plot in planar geometry
;    theta:      Requested rotation of plot in "theta" geometry
;    phi:        Requested rotation of plot in "phi" geometry
; OUTPUTS:
;    pxx         Actual X-positions for 2-D variable
;    pyy         Actual Y-positions for 2-D variable
;    pxx:        Used X-positions of data plotted
;    pyy:        Used Y-positions of data plotted
; OUTPUT KEYWORD PARAMETERS:
;    position    Actual range of positions [px0,py0,px1,py1]
;    position:   Used XY-positions [x0,y0,x1,y1] for plotting area
; COMMON BLOCKS:
;    None
; MODIFICATION HISTORY:
;    D. Odstrcil, Ondrejov, July 1994
;-
;===============================================================
pro position_set, x, y, pxx, pyy, xrange=xrange, yrange=yrange,$
  angle=angle, theta=theta, phi=phi, $
  pxrange=pxrange, pyrange=pyrange, position=position
;===============================================================
;---------------------------------------------------------------
;X-Y axes
  nx=n_elements(x)
  ny=n_elements(y)
  xx=fltarr(nx,ny)
  yy=fltarr(nx,ny)
;---------------------------------------------------------------
;X-Y coordinates
  case 1 of
    (n_elements(theta) ne 0): begin
      for ja=0,ny-1 do begin
        zcos=cos(!dtor*(y(ja)+theta))
        zsin=sin(!dtor*(y(ja)+theta))
        xx(*,ja)=x*zsin
        yy(*,ja)=x*zcos
      endfor
    end
    (n_elements(phi) ne 0): begin
      for ja=0,ny-1 do begin
        zcos=cos(!dtor*(y(ja)+phi))
        zsin=sin(!dtor*(y(ja)+phi))
        xx(*,ja)=x*zcos
        yy(*,ja)=x*zsin
      endfor
    end
    else: begin
      for jy=0,ny-1 do xx(*,jy)=x
      for jx=0,nx-1 do yy(jx,*)=y
    end
  endcase
;===============================================================
  if(n_elements(xrange) eq 0) then xrange=[min(x),max(x)]
  if(n_elements(yrange) eq 0) then yrange=[min(y),max(y)]
;---------------------------------------------------------------
;X-Y coordinates
  case 1 of
    (n_elements(theta) ne 0): begin
      position_max_theta, xrange, yrange, theta, $
	xmin, xmax, ymin, ymax
    end
    (n_elements(phi) ne 0): begin
      position_max_phi, xrange, yrange, phi, $
	xmin, xmax, ymin, ymax
    end
    else: begin
      if(not(keyword_set(angle))) then angle=0.
      position_max_planar, xrange, yrange, angle, $
	xmin, xmax, ymin, ymax
    end
  endcase
;===============================================================
;Save position-plot coordinates
;---------------------------------------------------------------
;X-offset(s) from X-positions
  if(keyword_set(pxrange)) then begin
    pxmin=pxrange(0)
    pxmax=pxrange(1)
  endif else begin
    pxmin=0.05
    pxmax=0.95
  endelse
;---------------------------------------------------------------
;Y-offset(s) from Y-positions
  if(keyword_set(pyrange)) then begin
    pymin=pyrange(0)
    pymax=pyrange(1)
  endif else begin
    pymin=0.05
    pymax=0.95
  endelse
;---------------------------------------------------------------
;Auxiliary values
  pxdif=float(pxmax-pxmin)
  pydif=float(pymax-pymin)
;---------------------------------------------------------------
;Actual size on a given device
  pxsize=pxdif*!d.x_size
  pysize=pydif*!d.y_size
;---------------------------------------------------------------
;Position range
  zrate=float(xmax-xmin)/float(ymax-ymin)
  if((pxsize/pysize) lt zrate) then begin
    pydif=pydif*(pxsize/pysize)/zrate
    pymax=pymin+pydif
  endif else begin
    pxdif=pxdif*(pysize/pxsize)*zrate
    pxmax=pxmin+pxdif
  endelse
;---------------------------------------------------------------
;Save position-range parameters
  position=[pxmin,pymin,pxmax,pymax]
;---------------------------------------------------------------
;Save position-plot coordinates
  zxrate=pxdif/float(xmax-xmin)
  zyrate=pydif/float(ymax-ymin)
  pxx=(xx-xmin)*zxrate+pxmin
  pyy=(yy-ymin)*zyrate+pymin
end
